# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""check gvn properties submitted to root of repository and rev properties.

This hook serves two purposes:
1) act on revision properties (currently gvn:bypass-hooks)
2) act on node properties set on the root of the repository

Currently this hook controls the gvn:superusers property that is attached
to the root of the repository.
You can set that property like this:
svn propset gvn:superusers group:svn-team .

Note that this means you can only set the superusers group before you install
this hook, or you'll have a chicken and egg problem :)
"""

import os
import re
import svn.core
import svn.fs
import gvn.userdb


SVN_USERDB_PATH = "conf/gvn/userdb"


def RunHook(hi, logger, userdb_path=None):
  """Implements gvn.hooks.runner's RunHook interface

  Additional arguments:
    userdb_path: when calling from unittest, allow another test userdb path

  Returns:
    -1: pass and bypass other hooks
    0: pass
    1: fail
    "string": fail and print string to the user
  """

  # If a committer or approver is in this special group, gvn:* property
  # changes on / are allowed
  gvn_revprops_override_group = hi.super_users

  if userdb_path is None:
    userdb_path = os.path.join(hi.repos_path, SVN_USERDB_PATH)

  userdb = gvn.userdb.UserDB(userdb_path)
  author = svn.fs.txn_prop(hi.txn, "svn:author", hi.pool)

  if svn.fs.txn_prop(hi.txn, "gvn:bypass-hooks") is not None:
    if userdb.UserInGroup(author, gvn_revprops_override_group):
      logger.info("Superuser %s invoked gvn:bypass-hooks" % author)
      return -1
    else:
      retstr = ("Non Superuser %s tried to invoke gvn:bypass-hooks" % author)
      logger.warn(retstr)
      return retstr

  # We only care about changes to the root of the repository
  if "/" not in hi.paths_changed:
    logger.debug("No changes to /, therefore no node properties to check, pass")
    return 0

  props = svn.fs.node_proplist(hi.txn_root, "/", hi.pool)
  logger.debug("got props %s for user %s" % (str(props), author))
  for property in props:
    logger.debug("Found property %s on repository root" % property)

    # You can add other forbidden properties here, just one for now
    if property != "gvn:superusers" or \
                userdb.UserInGroup(author, gvn_revprops_override_group):
      logger.debug("Allowing user %s to set/change property %s on "
                   "repository root" % (author, property))
      continue
    else:
      retstr= ("user %s cannot set/change property %s on repository root"
                                                    % (author, property))
      logger.warn(retstr)
      return retstr

  logger.info("all root properties changes accepted, success!")
  return 0
