# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import cPickle
import cStringIO
import unittest

import svn.fs

import gvn.project
import gvn.hooks.info


class HookInfoTests(unittest.TestCase):

  def testHookInfo_Basic(self):
    """Basic HookInfo tests.
    """
    hi = gvn.hooks.info.HookInfo(['bogus', 'repo'], userdb='ignored')
    self.assertEquals('bogus', hi.hook_name)
    self.assertEquals('repo', hi.repos_path)
    self.assertEquals('repo', hi.repos_name())
    self.assertEquals('svn.repo.hook.bogus', hi.logger_name())
    self.assertEquals('repo/hooks/bogus.d', hi.hook_dir)
    self.assertEquals(None, hi.action)
    self.assertEquals(None, hi.path)
    self.assertEquals(None, hi.propname)
    self.assertEquals(None, hi.propvalue)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals(None, hi.user)
    self.assertEquals(None, hi.locked_paths)

  # TODO(epg): Move into common test class.
  def assertInSequence(self, item, sequence, msg=None):
    if item not in sequence:
      if msg is None:
        msg = '%s not in %s' % (item, sequence)
      raise self.failureException(msg)

  def testHookInfo_project_config(self):
    hi = gvn.hooks.info.HookInfo(['bogus', 'repo'], userdb='ignored')
    # a test project_config dict
    test_config = dict(gvn.project._DEFAULT_CONFIG)
    test_config['diff-lines'] = 14
    # We don't care to test the cbb-processing, so drop this (and do
    # so in tests below).
    del test_config['change-branch-base']
    # Mock hi.head_root for testing.
    hi._head_root = object()
    # Mock svn.fs.node_prop for testing.
    def MockProp(root, path, propname, pool):
      self.assertInSequence(path, mock_prop_paths)
      self.assertEqual(propname, 'gvn:project')
      if path == project_path:
        return cPickle.dumps(test_config)
      return None
    orig = svn.fs.node_prop
    svn.fs.node_prop = MockProp
    try:
      # Test a change with no project config at all.
      project_path = None
      # Mock that the user changed /a and /b.
      hi._paths_changed = {'/a': None, '/b': None}
      # So MockProp should only be called for '' (repo root).
      mock_prop_paths = ['']
      # Assert that we found no project config, and thus have defaults.
      del hi.project_config['path']
      self.assertEqual(hi.project_config, gvn.project._DEFAULT_CONFIG)
      # Reset for next test.
      hi._prefix_changed = hi._project_config = None

      # Test a change with a project config on project/path.
      project_path = 'project/path'
      # Mock that the user changed these paths.
      hi._paths_changed = {'/project/path/a/b/c': None,
                           '/project/path/a/b/d/e': None,
                           '/project/path/a/b/d/e/f': None}
      # So MockProp should only be called for these paths.
      mock_prop_paths = ['project/path/a/b', 'project/path/a', 'project/path']
      # Assert that we found the proper project config.
      self.assertEqual(hi.project_config['path'], 'project/path')
      del hi.project_config['change-branch-base']
      del hi.project_config['path']
      self.assertEqual(hi.project_config, test_config)
    finally:
      # Restore svn.fs.node_prop.
      svn.fs.node_prop = orig

  def testHookInfo_PostCommit(self):
    argv = ['/repo/foo/hooks/post-commit', '/repo/foo', '42' ]
    hi = gvn.hooks.info.HookInfo(argv, userdb='ignored')
    self.assertEquals('post-commit', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.post-commit', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/post-commit.d', hi.hook_dir)
    self.assertEquals(None, hi.action)
    self.assertEquals(None, hi.path)
    self.assertEquals(None, hi.propname)
    self.assertEquals(None, hi.propvalue)
    self.assertEquals(42, hi.revision)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals(None, hi.user)
    self.assertEquals(None, hi.locked_paths)

  def testHookInfo_PostLock(self):
    argv = ['/repo/foo/hooks/post-lock', '/repo/foo', 'jrandom' ]
    stdin = """line1
line2"""
    hi = gvn.hooks.info.HookInfo(argv, cStringIO.StringIO(stdin),
                                 userdb='ignored')
    self.assertEquals('post-lock', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.post-lock', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/post-lock.d', hi.hook_dir)
    self.assertEquals(None, hi.action)
    self.assertEquals(None, hi.path)
    self.assertEquals(None, hi.propname)
    self.assertEquals(None, hi.propvalue)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals('jrandom', hi.user)
    self.assertEquals(['line1', 'line2'], hi.locked_paths)

  def testHookInfo_PostRevPropChange(self):
    argv = ['/repo/foo/hooks/post-revprop-change', '/repo/foo', '42',
            'jrandom', 'svn:keywords', 'A' ]
    stdin = 'HeadURL Id'
    hi = gvn.hooks.info.HookInfo(argv, cStringIO.StringIO(stdin),
                                 userdb='ignored')
    self.assertEquals('post-revprop-change', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.post-revprop-change', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/post-revprop-change.d', hi.hook_dir)
    self.assertEquals('A', hi.action)
    self.assertEquals(None, hi.path)
    self.assertEquals('svn:keywords', hi.propname)
    self.assertEquals(stdin, hi.propvalue)
    self.assertEquals(42, hi.revision)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals('jrandom', hi.user)
    self.assertEquals(None, hi.locked_paths)

  def testHookInfo_PostUnLock(self):
    argv = ['/repo/foo/hooks/post-unlock', '/repo/foo', 'jrandom' ]
    stdin = """line1
line2"""
    hi = gvn.hooks.info.HookInfo(argv, cStringIO.StringIO(stdin),
                                 userdb='ignored')
    self.assertEquals('post-unlock', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.post-unlock', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/post-unlock.d', hi.hook_dir)
    self.assertEquals(None, hi.action)
    self.assertEquals(None, hi.path)
    self.assertEquals(None, hi.propname)
    self.assertEquals(None, hi.propvalue)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals('jrandom', hi.user)
    self.assertEquals(['line1', 'line2'], hi.locked_paths)

  def testHookInfo_PreCommit(self):
    argv = ['/repo/foo/hooks/pre-commit', '/repo/foo', '42' ]
    hi = gvn.hooks.info.HookInfo(argv, userdb='ignored')
    self.assertEquals('pre-commit', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.pre-commit', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/pre-commit.d', hi.hook_dir)
    self.assertEquals(None, hi.action)
    self.assertEquals(None, hi.path)
    self.assertEquals(None, hi.propname)
    self.assertEquals(None, hi.propvalue)
    self.assertEquals('42', hi.txn_name)
    self.assertEquals(None, hi.user)
    self.assertEquals(None, hi.locked_paths)

  def testHookInfo_PreLock(self):
    argv = ['/repo/foo/hooks/pre-lock', '/repo/foo', '/path/2b/locked',
            'jrandom' ]
    hi = gvn.hooks.info.HookInfo(argv, userdb='ignored')
    self.assertEquals('pre-lock', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.pre-lock', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/pre-lock.d', hi.hook_dir)
    self.assertEquals(None, hi.action)
    self.assertEquals('/path/2b/locked', hi.path)
    self.assertEquals(None, hi.propname)
    self.assertEquals(None, hi.propvalue)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals('jrandom', hi.user)
    self.assertEquals(None, hi.locked_paths)

  def testHookInfo_PreRevPropChange(self):
    argv = ['/repo/foo/hooks/pre-revprop-change', '/repo/foo', '42',
            'jrandom', 'svn:keywords', 'A' ]
    stdin = 'HeadURL Id'
    hi = gvn.hooks.info.HookInfo(argv, cStringIO.StringIO(stdin),
                                 userdb='ignored')
    self.assertEquals('pre-revprop-change', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.pre-revprop-change', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/pre-revprop-change.d', hi.hook_dir)
    self.assertEquals('A', hi.action)
    self.assertEquals(None, hi.path)
    self.assertEquals('svn:keywords', hi.propname)
    self.assertEquals(stdin, hi.propvalue)
    self.assertEquals(42, hi.revision)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals('jrandom', hi.user)
    self.assertEquals(None, hi.locked_paths)

  def testHookInfo_PreUnLock(self):
    argv = ['/repo/foo/hooks/pre-unlock', '/repo/foo', '/path/2b/unlocked',
            'jrandom' ]
    hi = gvn.hooks.info.HookInfo(argv, userdb='ignored')
    self.assertEquals('pre-unlock', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.pre-unlock', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/pre-unlock.d', hi.hook_dir)
    self.assertEquals(None, hi.action)
    self.assertEquals('/path/2b/unlocked', hi.path)
    self.assertEquals(None, hi.propname)
    self.assertEquals(None, hi.propvalue)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals('jrandom', hi.user)
    self.assertEquals(None, hi.locked_paths)

  def testHookInfo_StartCommit(self):
    argv = ['/repo/foo/hooks/start-commit', '/repo/foo', 'jrandom' ]
    hi = gvn.hooks.info.HookInfo(argv, userdb='ignored')
    self.assertEquals('start-commit', hi.hook_name)
    self.assertEquals('/repo/foo', hi.repos_path)
    self.assertEquals('foo', hi.repos_name())
    self.assertEquals('svn.foo.hook.start-commit', hi.logger_name())
    self.assertEquals('/repo/foo/hooks/start-commit.d', hi.hook_dir)
    self.assertEquals(None, hi.action)
    self.assertEquals(None, hi.path)
    self.assertEquals(None, hi.propname)
    self.assertEquals(None, hi.propvalue)
    self.assertEquals(None, hi.txn_name)
    self.assertEquals('jrandom', hi.user)
    self.assertEquals(None, hi.locked_paths)
