# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import smtplib
import unittest

import gvn.config
import gvn.subcommands.mail

import testing.testcommon
import testing.test_changebranch


class TestMail(testing.test_changebranch.TestCase):

  def testParseMail(self):
    pconfig = gvn.config.ProjectConfig()
    before = ('from: basil\n'
           'to: manuel, major\n'
           '\n'
           'message body\n')
    after = ('from: basil\n'
             'to: manuel, major\n'
             'User-Agent: gvn/%s\n'
             'MIME-Version: 1.0\n'
             'Content-Type: text/plain; charset="utf-8"\n'
             'Content-Transfer-Encoding: 7bit\n'
             '\n'
             'message body\n' % (gvn.VERSION,))
    self.assertEquals(gvn.subcommands.mail.ParseMail(pconfig, before),
                      (['manuel', 'major'], after))
    before = ('from: basil\n'
              'to: manuel, major\n'
              'bcc: polly\n'
              'bcc: flyingtart\n'
              '\n'
              'body\n')
    after = ('from: basil\n'
             'to: manuel, major\n'
             'User-Agent: gvn/%s\n'
             'MIME-Version: 1.0\n'
             'Content-Type: text/plain; charset="utf-8"\n'
             'Content-Transfer-Encoding: 7bit\n'
             '\n'
             'body\n' % (gvn.VERSION,))
    self.assertEquals(gvn.subcommands.mail.ParseMail(pconfig, before),
                      (['manuel', 'major', 'polly', 'flyingtart'], after))

  def testHandle_GvnMail_Dash_cc(self):
    """Test that -cc is rejected without --force-change.

    '-cc' means '--change c' but is a common typo for '--cc'.
    """
    # Edit iota.
    fp = open('greek-tree/iota', 'a')
    fp.write('hi\n')
    testing.testcommon.sleepf(fp.close)

    # First test that -cc results in an error.
    options = ['--non-interactive', '-c', 'change1', 'greek-tree/iota',
               '--reviewers', 'terry', '-cc', 'guy i want to CC']
    ctx = gvn.cmdline.Context(*gvn.cmdline.OptionParser().parse_args(options))
    ctx.config = testing.testcommon.GetConfig()
    testing.testcommon.GetAuthBaton(ctx.config, ctx.pool,
                                    ctx.project.repository)
    self.assertRaises(gvn.errors.ChangeIsC,
                      gvn.subcommands.mail.Handle_GvnMail, ctx)


    # Now test that -cc with --force-change goes through.
    # Setup Context.
    options = ['--non-interactive', '-c', 'change1', 'greek-tree/iota',
               '--reviewers', 'terry', '-cc', '--force-change']
    ctx = gvn.cmdline.Context(*gvn.cmdline.OptionParser().parse_args(options))
    ctx.config = testing.testcommon.GetConfig()
    testing.testcommon.GetAuthBaton(ctx.config, ctx.pool,
                                    ctx.project.repository)
    # Mock smtplib.SMTP so this won't send mail; by the time it gets
    # there it's already created the changebranch anyway, which is
    # what we care about.
    class mock_SMTP(Exception):
      def __init__(self, *args, **kwargs):
        raise self
    orig_SMTP = smtplib.SMTP
    smtplib.SMTP = mock_SMTP
    try:
      # Mock ctx._stderr to grab the output.
      lines = []
      class mock_file(object):
        def write(self, line):
          lines.append(line)
        def flush(self): pass
      ctx._stderr = mock_file()
      # Now run and assert.
      self.assertRaises(mock_SMTP, gvn.subcommands.mail.Handle_GvnMail, ctx)
      self.assertEquals(lines,
                        ['Sending        %s\n' % (os.path.join('greek-tree',
                                                               'iota'),),
                         'Transmitting file data ', '.',
                         '\n',
                         ('Changed unittest/c@%d.\n'
                          % (ctx.project.repository.GetHead(),)),
                         'sending', '.\n'])
    finally:
      smtplib.SMTP = orig_SMTP
