# Copyright 2007 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import datetime
import os
import re
import unittest

import svn.core

from svn.core import SubversionException
from svn.core import SWIG_SVN_INVALID_REVNUM as SVN_INVALID_REVNUM

import gvn.util
import gvn.errors
import gvn.repository
from testing import testcommon
from testing.testcommon import RunSvn

DATE_RE = re.compile(r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d+Z$')

def IsValidDate(date):
  """Assert that the string 'date' looks like a valid date.

  NOTE: Not just any date format! Just 'year-mo-dyThr:mn:sc.extraZ'
  """
  return DATE_RE.match(date)

class TestRepository(unittest.TestCase):

  def setUp(self):
    self.config = testcommon.GetConfig()
    self.repo = testcommon.NewTmpRepository()
    self.repo_url = testcommon.TmpRepositoryURL()

    self.wcdir = testcommon.NewWorkingCopy()
    os.chdir(self.wcdir)
    self.assert_(os.path.isdir('./.svn'))
    self.pool = svn.core.Pool()

  def GetRepository(self):
    self.config.SetSvnConfig(os.path.join(os.environ['HOME'], '.subversion'), self.pool)

    repo = gvn.repository.Repository(testcommon.UnittestUser(),
                                     self.repo_url, self.config,
                                     self.pool)
    testcommon.GetAuthBaton(self.config, self.pool, repo)
    return repo

  def testInvalidConstructors(self):
    """try to make Repository from bogus URLs"""
    repo = self.GetRepository()

    # non-existent path in repo
    # TODO(epg): don't raise this generic thing
    self.assertRaises(SubversionException, repo.GetRevisions,
                      ['does/not/exist'], 1)

    # invalid URL
    # XXX(epg): This test depends on a bug-fix in svn r31223, which
    # I've nominated for backport to 1.5.1.  Ensure that it actually
    # makes it in!
    if svn.core.SVN_VER_MINOR == 5 and svn.core.SVN_VER_MICRO >= 1:
      repo = gvn.repository.Repository(testcommon.UnittestUser(),
                                       'iamfred', self.config,
                                       self.pool)
      testcommon.GetAuthBaton(self.config, self.pool, repo)
      self.assertRaises(SubversionException, repo.GetRevisions, [''], 1)

    # non-existent repo
    repo = gvn.repository.Repository(testcommon.UnittestUser(),
                                     self.repo_url[:-1],
                                     self.config, self.pool)
    testcommon.GetAuthBaton(self.config, self.pool, repo)
    self.assertRaises(SubversionException, repo.GetRevisions, [''], 1)

  def testShortToLongURL(self):
    repo = gvn.repository.Repository(url='https://server/path/dir1',
                      username=None, config=None, pool=None)
    repo.have_root_url = True
    self.assertEquals('https://server/path/dir1/dir2/name',
                      repo.ShortToLongURL('^/dir2/name'))
    self.assertEquals('https://server/path/dir1', repo.ShortToLongURL('^/'))
    self.assertRaises(AttributeError, repo.ShortToLongURL, None)
    self.assertRaises(AttributeError, repo.ShortToLongURL, 0)
    self.assertRaises(gvn.errors.NotShortURL, repo.ShortToLongURL, '')
    self.assertRaises(gvn.errors.NotShortURL, repo.ShortToLongURL,
                      '/hey/there')

  def testShortToLongURLCompat(self):
    repo = gvn.repository.Repository(url='https://server/path/dir1',
                      username=None, config=None, pool=None)
    repo.have_root_url = True
    self.assertEquals('https://server/path/dir1/dir2/name',
                      repo.ShortToLongURL('//dir2/name'))
    self.assertEquals('https://server/path/dir1', repo.ShortToLongURL('//'))
    self.assertRaises(AttributeError, repo.ShortToLongURL, None)
    self.assertRaises(AttributeError, repo.ShortToLongURL, 0)
    self.assertRaises(gvn.errors.NotShortURL, repo.ShortToLongURL, '')
    self.assertRaises(gvn.errors.NotShortURL, repo.ShortToLongURL,
                      '/hey/there')

  def testLongToShortURL(self):
    repo = gvn.repository.Repository(url='https://server/path/dir1',
                      username=None, config=None, pool=None)
    repo.have_root_url = True
    self.assertEquals('^/d2/name',
                      repo.LongToShortURL('https://server/path/dir1/d2/name'))
    self.assertEquals('^/', repo.LongToShortURL('https://server/path/dir1'))
    self.assertEquals('^/', repo.LongToShortURL('https://server/path/dir1/'))
    self.assertRaises(AttributeError, repo.LongToShortURL, None)
    self.assertRaises(AttributeError, repo.LongToShortURL, 0)
    self.assertRaises(gvn.errors.PathNotChild, repo.LongToShortURL, '')
    self.assertRaises(gvn.errors.PathNotChild, repo.LongToShortURL,
                      '/hey/there')

  def testGetRevisions(self):
    repo = self.GetRepository()

    revisions = repo.GetRevisions([''], 1)
    self.assertEquals(len(revisions), 1)
    self.assertEquals(revisions[0].number, 1)
    self.assertEquals(revisions[0]['svn:author'], 'unittest')
    self.assertEquals(len(revisions[0].paths), 5)

    # Log greek-tree.
    testdir = 'public/projects/greek-tree'
    revisions = repo.GetRevisions([testdir])
    # We don't know how many revisions it will have, as we're always
    # adding more to the dump file.
    (exitcode, out, err) = gvn.svncmd.CaptureSvn(['log', '-q',
                                                self.repo_url + '/' + testdir])
    self.assertEquals(exitcode, 0)
    n = len([x for x in out.splitlines() if x[0] == 'r'])
    self.assertEquals(len(revisions), n)
    # We know that the last in the list is r2, though.
    revision = revisions[-1]
    self.assertEquals(revision.number, 2)
    self.assertEquals(revision['svn:author'], 'unittest')
    try:
      log = revision['svn:log']
    except:
      log = None
    self.assertFalse(log == None)
    self.assertTrue(IsValidDate(revision['svn:date']))
    self.assertEquals(len(revision.paths), 21)
    for path in revision.paths:
      self.assertEquals(path.action, 'A')
      self.assertTrue(gvn.util.IsChild(path.path.lstrip('/'), testdir))

    # We know what revisions 2-4 look like.
    revisions = repo.GetRevisions([''], start=2, end=4)
    self.assertEquals(len(revisions), 3)
    self.assertEquals([x.number for x in revisions], [2, 3, 4])
    self.assertEquals([len(x.paths) for x in revisions], [21, 5, 5])
    self.assertEquals(revisions[0].paths[0].copyfrom_path, None)
    self.assertEquals(revisions[0].paths[0].copyfrom_revision,
                      SVN_INVALID_REVNUM)
